#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "avr_utils.h"
#include "key.h"
#include "lcd.h"

unsigned char ee_pwm_cal EEMEM = 0x6c;
unsigned char key_cal_x EEMEM = 21;
unsigned char key_cal_y EEMEM = 40;
unsigned char key_cal_x1 EEMEM = 237;
unsigned char key_cal_y1 EEMEM = 216;

volatile unsigned char key_cx;
volatile unsigned char key_cx1;
volatile unsigned char key_cy;
volatile unsigned char key_cy1;

volatile unsigned char key_val_chy;
volatile unsigned char key_val_chx;
volatile unsigned char key_val_state;
volatile unsigned char key_count;
volatile unsigned char key_pressed;

VoidFnct key_time_function=0;

void key_ex(void)
{
	process_keys();
	if(key_time_function) key_time_function();
}

void key_set_wait(void)
{
	SET(KEY_CH2B_D);
	OUTPUT(KEY_CH2B_D);

	CLR(KEY_CH1A_D);
	INPUT(KEY_CH1A_D);
	CLR(KEY_CH2A_D);
	INPUT(KEY_CH2A_D);

	CLR(KEY_CH1B_D);
	OUTPUT(KEY_CH1B_D);

	INPUT(KEY_CH2B_D);

	key_val_state=KEY_STATE_WAIT;
}

void key_init(void)
{

	// set all io's for Touch to out
	// adc enable und 128 als teilung
	ADCSRA=(1<<ADEN) | (7<<ADPS0);
	// erste channel einstellen und linksbuendig ausgeben (adch hat einen 8-bit-wert)
	ADMUX=(KEY_A_CH1<<MUX0) | (1<<ADLAR);

	key_set_wait();

	key_val_chx=0;
	key_val_chy=0;
	key_count=0;
	key_pressed=0;

	key_time_function=vsync_function;
	vsync_function=key_ex;

	key_cx=eeprom_read_byte(&key_cal_x);
	key_cx1=eeprom_read_byte(&key_cal_x1);
	key_cy=eeprom_read_byte(&key_cal_y);
	key_cy1=eeprom_read_byte(&key_cal_y1);
}

void key_set_cal(unsigned char x,unsigned char y,unsigned char x1,unsigned char y1)
{
	eeprom_write_byte(&key_cal_x, x); 
	eeprom_write_byte(&key_cal_y, y); 
	eeprom_write_byte(&key_cal_x1, x1); 
	eeprom_write_byte(&key_cal_y1, y1); 
	key_cx=eeprom_read_byte(&key_cal_x);
	key_cx1=eeprom_read_byte(&key_cal_x1);
	key_cy=eeprom_read_byte(&key_cal_y);
	key_cy1=eeprom_read_byte(&key_cal_y1);
}

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
unsigned char get_key_press( void )
{
	unsigned char a=key_pressed;
	if(a)
	{
		key_pressed=0;
	}
	return a;
}

unsigned int key_transform_x(void)
{
	long int a;

	a=LCD_DIMX;
	a=a*(key_val_chx-key_cx);
	a=a/(key_cx1-key_cx);
	if(a<0) a=0;
	if(a>LCD_DIMX) a=LCD_DIMX;

	return a;
}
  
unsigned int key_calc_x(unsigned char char_x)
{
	return char_x*LCD_WCHAR;
}
  
unsigned char get_key_pos( unsigned char *x,unsigned char *y )
{
	unsigned int a;

	a=(key_val_chx-key_cx)*KEY_X_CHARS;
	*x=a/(key_cx1-key_cx); 

	a=(key_val_chy-key_cy)*KEY_Y_CHARS;
	*y=a/(key_cy1-key_cy); 

	return 0;
}

unsigned char key_digital_pressed(void)
{
	if(key_val_state==KEY_STATE_WAIT)
	{
		// test if key is pressed
		if(!GET(KEY_CH2B_D))
		{
			return 1;
		}
	}
	return 0;
}

///////////////////////////////////////////////////////////////////
//
void process_keys( void )
{
	if(ADCSRA & (1<<ADSC))
	{
		// wandlung ist noch aktiv
		// keine auswertung ausfuehren
		return;
	}

	if(key_val_state==KEY_STATE_WAIT)
	{
		// test if key is pressed
		if(GET(KEY_CH2B_D))
		{
			key_count=0;
			return;
		}

		if(key_count>KEY_STORE)
		{
			if(key_count==KEY_R_START)
			{
				key_pressed=2;
			}

			key_count++;

			if(key_count>=KEY_R_NEXT)
			{
				key_count=KEY_R_START;
			}
			return;
		}
	}

	if(key_val_state==KEY_STATE_M1)
	{
		if(key_count==KEY_STORE) key_val_chy=ADCH;
	}
	
	if(key_val_state==KEY_STATE_M2)
	{
		if(key_count==KEY_STORE)
		{
			key_val_chx=ADCH;
			key_pressed=1;
		}
		key_set_wait();

		key_count++;
		return;
	}
	
	key_val_state++;

	switch(key_val_state)
	{
		case KEY_STATE_M2:
			CLR(KEY_CH1A_D);
			OUTPUT(KEY_CH1A_D);
			SET(KEY_CH1B_D);
			OUTPUT(KEY_CH1B_D);

			// hochohmig schalten (das ist der channel der gemessen wird)
			CLR(KEY_CH2A_D);
			INPUT(KEY_CH2A_D);
			CLR(KEY_CH2B_D);
			INPUT(KEY_CH2B_D);
			
			ADMUX=(KEY_A_CH2<<MUX0) | (1<<ADLAR);
			ADCSRA |= (1<<ADSC);
			break;

		case KEY_STATE_M1:
			// hochohmig schalten (das ist der channel der gemessen wird)
			CLR(KEY_CH1A_D);
			INPUT(KEY_CH1A_D);
			CLR(KEY_CH1B_D);
			INPUT(KEY_CH1B_D);

			SET(KEY_CH2A_D);
			OUTPUT(KEY_CH2A_D);
			CLR(KEY_CH2B_D);
			OUTPUT(KEY_CH2B_D);
			
			ADMUX=(KEY_A_CH1<<MUX0) | (1<<ADLAR);
			ADCSRA |= (1<<ADSC);
			break;
	}
}

#ifdef KEY_INIT_CAL
void key_init_contrast(void)
{
	unsigned char x,y;
	unsigned char tmp=0;

	// set contrast if touch is pressed while power on
	if(key_digital_pressed())
	{
		lcd_setscr();
		while( 1 )                                                          // Endlosschleife
		{
			if(get_key_press()==1)
			{
				lcd_gotoxy(0, 0);
				lcd_int(key_val_chx);
				lcd_gotoxy(0, 1);
				lcd_int(key_val_chy);
				lcd_gotoxy(x, y);
				lcd_putc(' ');
				get_key_pos(&x,&y);
				lcd_gotoxy(x, y);
				lcd_putc('#');
				if(y>15) break;
//				tmp=x*(255/KEY_X_CHARS);
				tmp=x*2+KEY_PWM_OFFSET;
				lcd_set_kontrast(tmp);
			}
		}                                                                   // end while( 1 )
		lcd_clrscr();
		eeprom_write_byte(&ee_pwm_cal, tmp); 
	}
}
#endif
