#ifndef MENU_H
#define MENU_H

#include <avr/io.h>
#include <avr/eeprom.h>
#include "avr_utils.h"

#define MENU_FUNCTION		0
#define MENU_RECT			1
#define MENU_RECT_DL		2
#define MENU_RECT_FILL		3
#define MENU_RECT_FILL_DL	4
#define MENU_RECT_BLOCK		5
#define MENU_TEXT			6
#define MENU_VAL			7
#define MENU_STRING			8
#define MENU_CHECK			9
#define MENU_RADIO			10
#define MENU_BUTTON			12
#define MENU_BUTTON_DL		13
#define MENU_MBUTTON		14
#define MENU_MBUTTON_DL		15
#define MENU_SLIDE			16

#ifdef LCD_MONSTER
#define MENU_MTEXT			26
#define MENU_MVAL			27
#endif


typedef void (*FuncPtr)(void);

typedef struct menu_rect 
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	unsigned char	w;					// width
	unsigned char	h;					// height
	FuncPtr			select;				// routine to call when selected 
} Menu_rect; 

typedef struct menu_text 
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	FuncPtr			select;				// routine to call when selected 
	char			*text;				// displayed entry 
} Menu_text; 

typedef struct menu_val 
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	int				*val;				// displayed value (EEMEM-Pointer)
	FuncPtr			select;				// routine to call when selected 
} Menu_val; 

typedef struct menu_string
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	char			len;				// len of value in eeprom
	char			*val;				// displayed value (EEMEM-Pointer)
	FuncPtr			select;				// routine to call when selected 
} Menu_string; 

typedef struct menu_button 
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	FuncPtr			select;				// routine to call when selected 
	char			*text;				// displayed entry 
} Menu_button; 

typedef struct menu_mbutton 
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	FuncPtr			select;				// routine to call when selected 
	struct menu_page	*page;				// page to call when selected 
	char			*text;				// displayed entry 
} Menu_mbutton; 

typedef struct menu_radio 
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	FuncPtr			select;				// routine to call when selected 
	char			*eep;				// eeprom-pointer
	char			*text[];			// displayed entrys 
} Menu_radio; 

typedef struct menu_slide
{ 
	unsigned char	func;				// functioncode
	unsigned char	x;					// x-pos
	unsigned char	y;					// y-pos
	unsigned char	w;					// width
	int				v_min;				// Min Value
	int				v_max;				// Max Value
	int				*eep;				// eeprom-pointer
	int				*tmp;				// tmp value
	FuncPtr			select;				// routine to call when selected 
} Menu_slide; 

typedef union
{
	Menu_rect		ptr_rect;
	Menu_text		ptr_text;
	Menu_button		ptr_button;
	Menu_mbutton	ptr_mbutton;
	Menu_val		ptr_val;
	Menu_radio		ptr_radio;
	Menu_slide		ptr_slide;
	Menu_string		ptr_string;
} m_ptr;

typedef struct menu_page 
{
	struct menu_page *next;
	m_ptr	*entries[];
} Menu_page; 


/*
** define mainmenu
*/

extern Menu_page MainMenu;

/*
** function prototypes
*/

void menu_draw_element(m_ptr *en);
void menu_draw_keypad(void);
void menu_keypad(char *txt, unsigned char len);
void menu_keypad_test(void);

void menu_init(void);
void menu_do(void);
void menu_do_post(void);
void menu_draw(void);

#endif
